@@ -20,7 +20,6 @@ package javax.tv.service.guide;
 
 import javax.tv.service.SIChangeListener;
 
-public interface ProgramScheduleListener extends SIChangeListener
-{
+public interface ProgramScheduleListener extends SIChangeListener {
     public void notifyChange(ProgramScheduleEvent event);
 }
