@@ -22,18 +22,9 @@ package org.havi.ui.event;
 import java.util.EventObject;
 
 public class HScreenLocationModifiedEvent extends EventObject {
-    public HScreenLocationModifiedEvent(Object source)
-    {
+    public HScreenLocationModifiedEvent(Object source) {
         super(source);
-
-        this.source = source;
     }
 
-    public Object getSource()
-    {
-        return source;
-    }
-
-    private Object source;
     private static final long serialVersionUID = -2915447586541903280L;
 }
