@@ -17,7 +17,6 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-
 package org.videolan;
 
 import java.io.FilePermission;
@@ -35,6 +34,15 @@ final class BDJSecurityManager extends SecurityManager {
     private String persistentRoot;
     private boolean usingUdf = false;
 
+    private static Class urlPermission = null;
+    static {
+        try {
+            /* Java 8 */
+            urlPermission = Class.forName("java.net.URLPermission");
+        } catch (Exception e) {
+        }
+    }
+
     BDJSecurityManager(String discRoot, String persistentRoot, String budaRoot) {
         this.discRoot  = discRoot;
         this.cacheRoot = null;
@@ -60,6 +68,18 @@ final class BDJSecurityManager extends SecurityManager {
      *
      */
 
+    private int classDepth0(String name) {
+        StackTraceElement e[] = new Exception("Stack trace").getStackTrace();
+        if (e != null && e.length > 1) {
+            for (int i = 1; i < e.length; i++) {
+                if (e[i].getClassName().equals(name)) {
+                    return i - 1;
+                }
+            }
+        }
+        return -1;
+    }
+
     private void deny(Permission perm) {
         logger.error("denied " + perm + "\n" + Logger.dumpStack());
         throw new SecurityException("denied " + perm);
@@ -68,10 +88,16 @@ final class BDJSecurityManager extends SecurityManager {
     public void checkPermission(Permission perm) {
         if (perm instanceof RuntimePermission) {
             if (perm.implies(new RuntimePermission("createSecurityManager"))) {
+
+                // allow initializing of javax.crypto.JceSecurityManager
+                if (classDepth0("javax.crypto.JceSecurityManager") < 3) {
+                    return;
+                }
+
                 deny(perm);
             }
             if (perm.implies(new RuntimePermission("setSecurityManager"))) {
-                if (classDepth("org.videolan.Libbluray") == 3) {
+                if (classDepth0("org.videolan.Libbluray") == 3) {
                     return;
                 }
                 deny(perm);
@@ -80,7 +106,7 @@ final class BDJSecurityManager extends SecurityManager {
             // work around bug in openjdk 7 / 8
             // sun.awt.AWTAutoShutdown.notifyThreadBusy is missing doPrivileged()
             // (fixed in jdk9 / http://hg.openjdk.java.net/jdk9/client/jdk/rev/5b613a3c04be )
-            if (classDepth("sun.awt.AWTAutoShutdown") > 0) {
+            if (classDepth0("sun.awt.AWTAutoShutdown") > 0) {
                 return;
             }
 
@@ -139,6 +165,11 @@ final class BDJSecurityManager extends SecurityManager {
                 return;
             }
         }
+        else if (urlPermission != null &&
+                 urlPermission.isInstance(perm)) {
+            logger.info("grant " + perm);
+            return;
+        }
 
         /* Java TV */
         else if (perm instanceof javax.tv.service.ReadPermission) {
@@ -170,6 +201,14 @@ final class BDJSecurityManager extends SecurityManager {
             return;
         }
 
+        else if (perm instanceof java.awt.AWTPermission) {
+            /* silence failures from clipboard access */
+            if (perm.getName().equals("accessClipboard")) {
+                java.security.AccessController.checkPermission(perm);
+                return;
+            }
+        }
+
         try {
             java.security.AccessController.checkPermission(perm);
         } catch (java.security.AccessControlException ex) {
@@ -178,6 +217,31 @@ final class BDJSecurityManager extends SecurityManager {
         }
     }
 
+    /*
+     * Allow package access (Java 11)
+     */
+
+    private static String pkgPrefixes[] = {
+        "javax.media" ,
+        "javax.tv",
+        "javax.microedition",
+        "org.davic",
+        "org.dvb",
+        "org.havi",
+        "org.bluray",
+        "org.blurayx",
+        "com.aacsla.bluray",
+    };
+
+    public void checkPackageAccess(String pkg) {
+
+        for (int i = 0; i < pkgPrefixes.length; i++)
+            if (pkg.startsWith(pkgPrefixes[i]))
+                return;
+
+        super.checkPackageAccess(pkg);
+    }
+
     /*
      *
      */
@@ -257,8 +321,7 @@ final class BDJSecurityManager extends SecurityManager {
         throw new SecurityException("write access denied");
     }
 
-    private String getCanonPath(String origPath)
-    {
+    private String getCanonPath(String origPath) {
         String suffix = "";
 
         if (!java.io.BDFileSystem.isAbsolutePath(origPath)) {
